package com.abi.qms.platform.controller.console;

import com.abi.qms.platform.dto.req.GetQrVariableDetailReq;
import com.abi.qms.platform.dto.req.ListQrVariableReq;
import com.abi.qms.platform.dto.req.PreviewQrVariableReq;
import com.abi.qms.platform.dto.res.GetQrVariableDetailRes;
import com.abi.qms.platform.dto.res.ListQrVariableRes;
import com.abi.qms.platform.dto.res.PreviewQrVariableRes;
import com.abi.qms.platform.service.QrVariableService;
import com.abi.task.common.api.base.BaseResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 码格式变量 Controller
 * 
 * @author WeiganCai
 * @date 2021-04-19
 */
@Slf4j
@RestController
@Api(tags = "码格式变量管理")
@RequestMapping("/qrVariable")
public class QrVariableController {

    @Autowired
    private QrVariableService qrVariableService;

    @ApiOperation("码格式变量查询分页")
    @GetMapping("/listQrVariable")
    public BaseResponse<ListQrVariableRes> listQrVariable(@Validated ListQrVariableReq listQrVariableReq) {
        ListQrVariableRes result = qrVariableService.listQrVariable(listQrVariableReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码格式变量生成预览")
    @GetMapping("/previewQrVariable")
    public BaseResponse<PreviewQrVariableRes> previewQrVariable(@Validated PreviewQrVariableReq previewQrVariableReq) {
        PreviewQrVariableRes result = qrVariableService.previewQrVariable(previewQrVariableReq);
        return BaseResponse.create(result);
    }

    @ApiOperation("码格式变量查询详情")
    @GetMapping("/getQrVariableDetail")
    public BaseResponse getQrVariableDetail(@Validated GetQrVariableDetailReq getQrVariableDetailReq) {
        GetQrVariableDetailRes result = qrVariableService.getQrVariableDetail(getQrVariableDetailReq);
        return BaseResponse.create(result);
    }

}
