package com.abi.qms.platform.infrastructure.interceptor;

import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.infrastructure.exception.ErrorCodeEnum;
import com.abi.qms.platform.dto.req.TokenReq;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.LoginService;
import com.abi.task.common.annotation.IgnoreToken;
import com.abi.task.common.api.exception.BusinessException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author zd
 * @date 2021/1/20 17:57
 */
@Configuration
public class TokenInterceptor extends HandlerInterceptorAdapter {

    @Resource
    private LoginService loginService;
    
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        IgnoreToken ignoreToken=null;
        if (handler instanceof HandlerMethod) {
            ignoreToken = ((HandlerMethod) handler).getMethodAnnotation(IgnoreToken.class);
        }else {
            return true;
        }

        if(ignoreToken != null){
            return true;
        }
        String token = request.getHeader("Authorization");
        if(StringUtils.isBlank(token)){
            token = request.getParameter("auth_token");
        }

        TokenReq tokenReq = new TokenReq();
        tokenReq.setToken(token);
        UserInfo userInfo = loginService.validateToken(tokenReq);
        if(userInfo != null){
            UserUtil.setUser(userInfo);
        }else{
            throw new BusinessException(ErrorCodeEnum.TOKEN_TIME_OUT.getCode(),ErrorCodeEnum.TOKEN_TIME_OUT.getMessage());
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        UserUtil.releaseUser();
    }
}
