package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.BoxCodeFormat;
import com.abi.qms.platform.dao.enums.StateEnum;
import com.abi.qms.platform.dao.mapper.BoxCodeFormatMapper;
import com.abi.qms.platform.dao.vo.result.BoxCodeFormatVO;
import com.abi.qms.platform.dto.req.BoxCodeFormatDetailReq;
import com.abi.qms.platform.dto.req.DeleteBoxCodeFormatreq;
import com.abi.qms.platform.dto.req.DisableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.EnableBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.ListBoxCodeFormatReq;
import com.abi.qms.platform.dto.req.SaveBoxCodeFormatReq;
import com.abi.qms.platform.dto.res.BoxCodeFormatDetailRes;
import com.abi.qms.platform.dto.res.ListBoxCodeFormatRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.BoxCodeFormatService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.api.exception.ErrorCodeEnum;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author: fangxinjian
 * @date: 2021/04/20 09:49
 * @description: 箱码幅面管理
 */
@Service
@Slf4j
public class BoxCodeFormatImpl implements BoxCodeFormatService {

    @Autowired
    private BoxCodeFormatMapper boxCodeFormatMapper;

    @Override
    public void saveBoxCodeFormat(SaveBoxCodeFormatReq req) {

        //1-声明对象
        BoxCodeFormat boxCodeFormat = null;
        if (req.getId() != null) {
            boxCodeFormat = boxCodeFormatMapper.selectById(req.getId());
            AssertUtil.isNull(boxCodeFormat, "箱码幅面ID不存在");
        } else {
            boxCodeFormat = new BoxCodeFormat();
        }

        //2-放入数据
        boxCodeFormat.setDeptId(req.getDeptId());
        boxCodeFormat.setCodeStoreId(req.getCodeStoreId());
        boxCodeFormat.setFactoryId(req.getFactoryId());
        boxCodeFormat.setName(req.getName());
        boxCodeFormat.setColorPerRollScalar(req.getColorPerRollScalar());
        boxCodeFormat.setFormatNumber(req.getFormatNumber());
        boxCodeFormat.setRemark(req.getRemark());

        //3-新增or修改
        if (req.getId() != null) {
            boxCodeFormatMapper.updateById(boxCodeFormat);
        } else {
            boxCodeFormatMapper.insert(boxCodeFormat);
        }

    }

    @Override
    public ListBoxCodeFormatRes listBoxCodeFormat(ListBoxCodeFormatReq req) {

        //分页查询
        IPage<BoxCodeFormatVO> iPage = boxCodeFormatMapper.listBoxCodeFormat(PageUtil.createPage(req), req);
        List<BoxCodeFormatVO> formatList = iPage.getRecords();
        //封装出参
        ListBoxCodeFormatRes res = new ListBoxCodeFormatRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListBoxCodeFormatRes.BoxCodeFormatBean> beanList = PojoConverterUtils.copyList(formatList, ListBoxCodeFormatRes.BoxCodeFormatBean.class);
        res.setBoxCodeFormatBeanList(beanList);

        return res;
    }

    @Override
    public BoxCodeFormatDetailRes getBoxCodeFormatDetail(BoxCodeFormatDetailReq req) {

        //查询箱码幅面详情
        BoxCodeFormatVO boxCodeFormatVO = boxCodeFormatMapper.selectBoxCodeFormatDetailById(req.getId());
        AssertUtil.isNull(boxCodeFormatVO, "箱码幅面详情不存在");

        //转化出参
        BoxCodeFormatDetailRes res = PojoConverterUtils.copy(boxCodeFormatVO, BoxCodeFormatDetailRes.class);

        return res;
    }

    @Override
    public void enableBoxCodeFormat(EnableBoxCodeFormatReq req) {

        List<Long> ids = req.getIds();

        //循环启用
        for (Long id : ids) {
            //1-查询活动对象
            BoxCodeFormat boxCodeFormat = boxCodeFormatMapper.selectById(id);
            AssertUtil.isNull(boxCodeFormat, "箱码幅面不存在");
            if (!StateEnum.DISABLE.is(boxCodeFormat.getState())) {
                continue;
            }

            //2-修改状态
            boxCodeFormat.setState(StateEnum.ENABLE.getCode());
            boxCodeFormatMapper.updateById(boxCodeFormat);
        }
    }

    @Override
    public void disableBoxCodeFormat(DisableBoxCodeFormatReq req) {

        List<Long> ids = req.getIds();

        //循环禁用
        for (Long id : ids) {
            //1-查询活动对象
            BoxCodeFormat boxCodeFormat = boxCodeFormatMapper.selectById(id);
            AssertUtil.isNull(boxCodeFormat, "箱码幅面不存在");
            if (!StateEnum.ENABLE.is(boxCodeFormat.getState())) {
                continue;
            }

            //2-修改状态
            boxCodeFormat.setState(StateEnum.DISABLE.getCode());
            boxCodeFormatMapper.updateById(boxCodeFormat);
        }

    }

    @Override
    public void deleteBoxCodeFormat(DeleteBoxCodeFormatreq req) {

        //查询箱码幅面
        BoxCodeFormat boxCodeFormat = boxCodeFormatMapper.selectById(req.getId());
        if (boxCodeFormat == null) {
            throw new BusinessException(ErrorCodeEnum.NO_EXISTS_DATA.getCode(), "箱码幅面不存在");
        }

        //逻辑删除
        boxCodeFormat.setIsDelete(1);
        boxCodeFormatMapper.updateById(boxCodeFormat);
    }


}
