package com.abi.qms.platform;

import com.abi.qms.platform.dao.entity.QrData;
import com.abi.qms.platform.dao.entity.QrInnerData;
import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.mapper.QrPackageMapper;
import com.abi.qms.platform.infrastructure.mq.GenerateCodeConsumer;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.mongodb.core.MongoTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * 生成码测试
 *
 * @author WeiganCai
 * @date: 2021-05-08
 */
@SpringBootTest
@Slf4j
public class GenerateCodeTest {
	@Autowired
	private AmqpTemplate amqpTemplate;

	@Autowired
	private QrPackageMapper qrPackageMapper;

	@Autowired
	private MongoTemplate mongoTemplate;

	@Test
	public void generateCodeTest() {
		QrPackage qrPackage = qrPackageMapper.selectById(3);
		amqpTemplate.convertAndSend(GenerateCodeConsumer.GENERATE_CODE_BY_REPERTORY_QUEUE, qrPackage);
	}

	@Test
	public void uniqueTest() {
		QrData qrData = new QrData();
		List<QrInnerData> innerDataList = new ArrayList<>();
		innerDataList.add(new QrInnerData().setCode("123456"));
		innerDataList.add(new QrInnerData().setCode("123456"));
		qrData.setInnerDataList(innerDataList);
		mongoTemplate.insert(qrData);
	}

	@Test
	public void uniqueListTest() {
		List<QrData> qrDataList = new ArrayList<>();

		QrData qrData = new QrData();
		List<QrInnerData> innerDataList = new ArrayList<>();
		innerDataList.add(new QrInnerData().setCode("789"));
		innerDataList.add(new QrInnerData().setCode("1011"));
		qrData.setInnerDataList(innerDataList);
		qrDataList.add(qrData);

		qrData = new QrData();
		innerDataList = new ArrayList<>();
		innerDataList.add(new QrInnerData().setCode("xyz"));
		innerDataList.add(new QrInnerData().setCode("789"));
		qrData.setInnerDataList(innerDataList);
		qrDataList.add(qrData);

		mongoTemplate.insert(qrDataList, QrData.class);
	}
}
